@@ -11,15 +11,16 @@
 #' Non-stepwise selection can be slow, especially for seasonal data. The stepwise
 #' algorithm outlined in Hyndman & Khandakar (2008) is used except that the default
 #' method for selecting seasonal differences is now based on an estimate of seasonal
-#' strength (Wang, Smith & Hyndman, 2006) rather than the Canova-Hansen test. 
-#' There are also some other minor variations to the algorithm described in 
+#' strength (Wang, Smith & Hyndman, 2006) rather than the Canova-Hansen test.
+#' There are also some other minor variations to the algorithm described in
 #' Hyndman and Khandakar (2008).
 #'
+#' @inheritParams stats::arima
 #' @param y a univariate time series
 #' @param d Order of first-differencing. If missing, will choose a value based
 #' on KPSS test.
 #' @param D Order of seasonal-differencing. If missing, will choose a value
-#' based on OCSB test.
+#' based on \code{season.test}.
 #' @param max.p Maximum value of p
 #' @param max.q Maximum value of q
 #' @param max.P Maximum value of P
@@ -38,6 +39,7 @@
 #' @param stepwise If \code{TRUE}, will do stepwise selection (faster).
 #' Otherwise, it searches over all models. Non-stepwise selection can be very
 #' slow, especially for seasonal models.
+#' @param nmodels Maximum number of models considered in the stepwise search.
 #' @param trace If \code{TRUE}, the list of ARIMA models considered will be
 #' reported.
 #' @param approximation If \code{TRUE}, estimation is via conditional sums of
@@ -50,11 +52,16 @@
 #' select a model when \code{truncate} is not \code{NULL} and
 #' \code{approximation=TRUE}. All observations are used if either
 #' \code{truncate=NULL} or \code{approximation=FALSE}.
-#' @param xreg Optionally, a vector or matrix of external regressors, which
-#' must have the same number of rows as \code{y}.
+#' @param xreg Optionally, a numerical vector or matrix of external regressors, which
+#' must have the same number of rows as \code{y}. (It should not be a data frame.)
 #' @param test Type of unit root test to use. See \code{\link{ndiffs}} for
 #' details.
-#' @param seasonal.test This determines which seasonal unit root test is used.
+#' @param test.args Additional arguments to be passed to the unit root test.
+#' @param seasonal.test This determines which method is used to select the number of seasonal differences.
+#' The default method is to use a measure of seasonal strength computed from an STL decomposition.
+#' Other possibilities involve seasonal unit root tests.
+#' @param seasonal.test.args Additional arguments to be passed to the seasonal
+#' unit root test.
 #' See \code{\link{nsdiffs}} for details.
 #' @param allowdrift If \code{TRUE}, models with drift terms are considered.
 #' @param allowmean If \code{TRUE}, models with a non-zero mean are considered.
@@ -68,14 +75,14 @@
 #' @param x Deprecated. Included for backwards compatibility.
 #' @param ... Additional arguments to be passed to \code{\link[stats]{arima}}.
 #' @inheritParams forecast
-#' 
+#'
 #' @return Same as for \code{\link{Arima}}
 #' @author Rob J Hyndman
 #' @seealso \code{\link{Arima}}
 #' @references Hyndman, RJ and Khandakar, Y (2008) "Automatic time series
 #' forecasting: The forecast package for R", \emph{Journal of Statistical
 #' Software}, \bold{26}(3).
-#' 
+#'
 #' Wang, X, Smith, KA, Hyndman, RJ (2006) "Characteristic-based clustering
 #' for time series data", \emph{Data Mining and Knowledge Discovery},
 #' \bold{13}(3), 335-364.
@@ -89,10 +96,11 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
                        max.P=2, max.Q=2, max.order=5, max.d=2, max.D=1,
                        start.p=2, start.q=2, start.P=1, start.Q=1,
                        stationary=FALSE, seasonal=TRUE, ic=c("aicc", "aic", "bic"),
-                       stepwise=TRUE, trace=FALSE,
+                       stepwise=TRUE, nmodels = 94, trace=FALSE,
                        approximation=(length(x) > 150 | frequency(x) > 12),
-                       truncate=NULL, xreg=NULL,
-                       test=c("kpss", "adf", "pp"), seasonal.test=c("seas", "ocsb", "hegy", "ch"),
+                       method = NULL, truncate=NULL, xreg=NULL,
+                       test=c("kpss", "adf", "pp"), test.args = list(),
+                       seasonal.test=c("seas", "ocsb", "hegy", "ch"), seasonal.test.args = list(),
                        allowdrift=TRUE, allowmean=TRUE, lambda=NULL, biasadj=FALSE,
                        parallel=FALSE, num.cores=2, x=y, ...) {
   # Only non-stepwise parallel implemented so far.
@@ -100,7 +108,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     warning("Parallel computer is only implemented when stepwise=FALSE, the model will be fit in serial.")
     parallel <- FALSE
   }
-  
+
   if (trace && parallel) {
     message("Tracing model searching in parallel is not supported.")
     trace <- FALSE
@@ -112,6 +120,16 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     stop("auto.arima can only handle univariate time series")
   }
 
+  # Trim leading NAs and find length of non-missing data
+  orig.x <- x
+  missing <- is.na(x)
+  firstnonmiss <- head(which(!missing),1)
+  lastnonmiss <- tail(which(!missing),1)
+  serieslength <- lastnonmiss - firstnonmiss + 1
+
+  # Trim initial missing values
+  x <- subset(x, start=firstnonmiss)
+
   # Check for constant data
   if (is.constant(x)) {
     if(all(is.na(x)))
@@ -121,7 +139,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     } else {
       fit <- Arima(x, order = c(0, 0, 0), include.mean = FALSE, ...)
     }
-    fit$x <- x
+    fit$x <- orig.x
     fit$series <- series
     fit$call <- match.call()
     fit$call$x <- data.frame(x = x)
@@ -132,13 +150,6 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   test <- match.arg(test)
   seasonal.test <- match.arg(seasonal.test)
 
-  # Use AIC if npar <= 3
-  # AICc won't work for tiny samples.
-  serieslength <- length(x)
-  if (serieslength <= 3L) {
-    ic <- "aic"
-  }
-
   # Only consider non-seasonal models
   if (seasonal) {
     m <- frequency(x)
@@ -158,8 +169,13 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   max.P <- min(max.P, floor(serieslength / 3 / m))
   max.Q <- min(max.Q, floor(serieslength / 3 / m))
 
-  orig.x <- x
+  # Use AIC if npar <= 3
+  # AICc won't work for tiny samples.
+  if (serieslength <= 3L) {
+    ic <- "aic"
+  }
 
+  # Transform data if requested
   if (!is.null(lambda)) {
     x <- BoxCox(x, lambda)
     lambda <- attr(x, "lambda")
@@ -169,6 +185,8 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   # Check xreg and do regression if necessary
   if (!is.null(xreg)) {
     # Make sure it is a matrix with column names
+    if("data.frame" %in% class(xreg))
+      stop("xreg should be a numeric matrix or vector")
     nmxreg <- deparse(substitute(xreg))
     xregg <- as.matrix(xreg)
     if (ncol(xregg) == 1 && length(nmxreg) > 1) {
@@ -182,24 +200,29 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
       }
     }
 
+    xx <- x
     # Check that xreg is not rank deficient
     # First check if any columns are constant
     constant_columns <- apply(xregg, 2, is.constant)
-    if (any(constant_columns)) { # Remove first one
-      xregg <- xregg[, -which(constant_columns)[1]]
+    if (all(constant_columns)) {
+      xregg <- NULL
     }
+    else{
+      if (any(constant_columns)) {
+        xregg <- xregg[, -which(constant_columns)]
+      }
 
-    # Now check if it is rank deficient
-    sv <- svd(na.omit(cbind(rep(1, NROW(xregg)), xregg)))$d
-    if (min(sv) / sum(sv) < .Machine$double.eps) {
-      stop("xreg is rank deficient")
+      # Now check if it is rank deficient
+      sv <- svd(na.omit(cbind(rep(1, NROW(xregg)), xregg)))$d
+      if (min(sv) / sum(sv) < .Machine$double.eps) {
+        stop("xreg is rank deficient")
+      }
+
+      # Finally find residuals from regression in order
+      # to estimate appropriate level of differencing
+      j <- !is.na(x) & !is.na(rowSums(xregg))
+      xx[j] <- residuals(lm(x ~ xregg))
     }
-
-    # Finally find residuals from regression in order
-    # to estimate appropriate level of differencing
-    j <- !is.na(x) & !is.na(rowSums(xregg))
-    xx <- x
-    xx[j] <- residuals(lm(x ~ xregg))
   }
   else {
     xx <- x
@@ -215,7 +238,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   }
   else if(is.na(D))
   {
-    D <- nsdiffs(xx, test=seasonal.test, max.D=max.D)
+    D <- do.call("nsdiffs", c(list(xx, test=seasonal.test, max.D=max.D), seasonal.test.args))
     # Make sure xreg is not null after differencing
     if (D > 0 && !is.null(xregg)) {
       diffxreg <- diff(xregg, differences = D, lag = m)
@@ -223,6 +246,12 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
         D <- D - 1
       }
     }
+    # Make sure xx is not all missing after differencing
+    if (D > 0) {
+      dx <- diff(xx, differences = D, lag = m)
+      if (all(is.na(dx)))
+        D <- D - 1
+    }
   }
   if (D > 0) {
     dx <- diff(xx, differences = D, lag = m)
@@ -237,7 +266,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     }
   }
   if (is.na(d)) {
-    d <- ndiffs(dx, test = test, max.d = max.d)
+    d <- do.call("ndiffs", c(list(dx, test = test, max.d = max.d), test.args))
     # Make sure xreg is not null after differencing
     if (d > 0 && !is.null(xregg)) {
       diffxreg <- diff(diffxreg, differences = d, lag = 1)
@@ -245,6 +274,12 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
         d <- d - 1
       }
     }
+    # Make sure dx is not all missing after differencing
+    if (d > 0) {
+      diffdx <- diff(dx, differences=d, lag=1)
+      if(all(is.na(diffdx)))
+        d <- d - 1
+    }
   }
 
   # Check number of differences selected
@@ -261,9 +296,9 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   if (is.constant(dx)) {
     if (is.null(xreg)) {
       if (D > 0) {
-        fit <- Arima(x, order = c(0, d, 0), seasonal = list(order = c(0, D, 0), period = m), include.constant = TRUE, fixed = mean(dx, na.rm = TRUE), ...)
+        fit <- Arima(x, order = c(0, d, 0), seasonal = list(order = c(0, D, 0), period = m), include.constant = TRUE, fixed = mean(dx/m, na.rm = TRUE), method = method, ...)
       } else if (d < 2) {
-        fit <- Arima(x, order = c(0, d, 0), include.constant = TRUE, fixed = mean(dx, na.rm = TRUE), ...)
+        fit <- Arima(x, order = c(0, d, 0), include.constant = TRUE, fixed = mean(dx, na.rm = TRUE), method = method, ...)
       } else {
         stop("Data follow a simple polynomial and are not suitable for ARIMA modelling.")
       }
@@ -271,9 +306,9 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     else # Perfect regression
     {
       if (D > 0) {
-        fit <- Arima(x, order = c(0, d, 0), seasonal = list(order = c(0, D, 0), period = m), xreg = xreg, ...)
+        fit <- Arima(x, order = c(0, d, 0), seasonal = list(order = c(0, D, 0), period = m), xreg = xreg, method = method, ...)
       } else {
-        fit <- Arima(x, order = c(0, d, 0), xreg = xreg, ...)
+        fit <- Arima(x, order = c(0, d, 0), xreg = xreg, method = method, ...)
       }
     }
     fit$x <- orig.x
@@ -324,15 +359,15 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   allowmean <- allowmean & (d + D) == 0
 
   constant <- allowdrift | allowmean
-  
+
   if (approximation && trace) {
     cat("\n Fitting models using approximations to speed things up...\n")
   }
-  
+
   if (!stepwise) {
     bestfit <- search.arima(
       x, d, D, max.p, max.q, max.P, max.Q, max.order, stationary,
-      ic, trace, approximation, xreg = xreg, offset = offset,
+      ic, trace, approximation, method = method, xreg = xreg, offset = offset,
       allowdrift = allowdrift, allowmean = allowmean,
       parallel = parallel, num.cores = num.cores, ...
     )
@@ -341,7 +376,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     bestfit$lambda <- lambda
     bestfit$x <- orig.x
     bestfit$series <- series
-    bestfit$fitted <- fitted(bestfit)
+    bestfit$fitted <- fitted.Arima(bestfit)
     if (trace) {
       cat("\n\n Best model:", arima.string(bestfit, padding = TRUE), "\n\n")
     }
@@ -360,171 +395,229 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   P <- start.P <- min(start.P, max.P)
   Q <- start.Q <- min(start.Q, max.Q)
 
-  results <- matrix(NA, nrow = 100, ncol = 8)
+  results <- matrix(NA, nrow = nmodels, ncol = 8)
 
-  bestfit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+  bestfit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
   results[1, ] <- c(p, d, q, P, D, Q, constant, bestfit$ic)
   # Null model with possible constant
-  fit <- myarima(x, order = c(0, d, 0), seasonal = c(0, D, 0), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+  fit <- myarima(x, order = c(0, d, 0), seasonal = c(0, D, 0), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
   results[2, ] <- c(0, d, 0, 0, D, 0, constant, fit$ic)
   if (fit$ic < bestfit$ic) {
     bestfit <- fit
     p <- q <- P <- Q <- 0
   }
+  k <- 2
   # Basic AR model
   if (max.p > 0 || max.P > 0) {
-    fit <- myarima(x, order = c(max.p > 0, d, 0), seasonal = c((m > 1) & (max.P > 0), D, 0), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
-    results[3, ] <- c(1, d, 0, m > 1, D, 0, constant, fit$ic)
+    fit <- myarima(x, order = c(max.p > 0, d, 0), seasonal = c((m > 1) & (max.P > 0), D, 0), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+    results[k+1, ] <- c(max.p > 0, d, 0, (m > 1) & (max.P > 0), D, 0, constant, fit$ic)
     if (fit$ic < bestfit$ic) {
       bestfit <- fit
       p <- (max.p > 0)
       P <- (m > 1) & (max.P > 0)
       q <- Q <- 0
     }
+    k <- k + 1
   }
   # Basic MA model
   if (max.q > 0 || max.Q > 0) {
-    fit <- myarima(x, order = c(0, d, max.q > 0), seasonal = c(0, D, (m > 1) & (max.Q > 0)), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
-    results[4, ] <- c(0, d, 1, 0, D, m > 1, constant, fit$ic)
+    fit <- myarima(x, order = c(0, d, max.q > 0), seasonal = c(0, D, (m > 1) & (max.Q > 0)), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+    results[k+1, ] <- c(0, d, max.q > 0, 0, D, (m > 1) & (max.Q > 0), constant, fit$ic)
     if (fit$ic < bestfit$ic) {
       bestfit <- fit
       p <- P <- 0
       Q <- (m > 1) & (max.Q > 0)
       q <- (max.q > 0)
     }
+    k <- k + 1
   }
-  k <- 4
   # Null model with no constant
   if (constant) {
-    fit <- myarima(x, order = c(0, d, 0), seasonal = c(0, D, 0), constant = FALSE, ic, trace, approximation, offset = offset, xreg = xreg, ...)
-    results[5, ] <- c(0, d, 0, 0, D, 0, 0, fit$ic)
+    fit <- myarima(x, order = c(0, d, 0), seasonal = c(0, D, 0), constant = FALSE, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+    results[k+1, ] <- c(0, d, 0, 0, D, 0, 0, fit$ic)
     if (fit$ic < bestfit$ic) {
       bestfit <- fit
       p <- q <- P <- Q <- 0
     }
-    k <- 5
+    k <- k + 1
   }
 
   startk <- 0
-  while (startk < k && k < 94) {
+  while (startk < k && k < nmodels) {
     startk <- k
     if (P > 0 && newmodel(p, d, q, P - 1, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q), seasonal = c(P - 1, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P - 1, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q, P - 1, D, Q, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         P <- (P - 1)
-      }
-    }
-    if (P < max.P && newmodel(p, d, q, P + 1, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q), seasonal = c(P + 1, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
-      results[k, ] <- c(p, d, q, P + 1, D, Q, constant, fit$ic)
-      if (fit$ic < bestfit$ic) {
-        bestfit <- fit
-        P <- (P + 1)
+        next
       }
     }
     if (Q > 0 && newmodel(p, d, q, P, D, Q - 1, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q - 1), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q - 1), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q, P, D, Q - 1, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         Q <- (Q - 1)
+        next
+      }
+    }
+    if (P < max.P && newmodel(p, d, q, P + 1, D, Q, constant, results[1:k, ])) {
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P + 1, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+      results[k, ] <- c(p, d, q, P + 1, D, Q, constant, fit$ic)
+      if (fit$ic < bestfit$ic) {
+        bestfit <- fit
+        P <- (P + 1)
+        next
       }
     }
     if (Q < max.Q && newmodel(p, d, q, P, D, Q + 1, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q + 1), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q + 1), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q, P, D, Q + 1, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         Q <- (Q + 1)
+        next
       }
     }
     if (Q > 0 && P > 0 && newmodel(p, d, q, P - 1, D, Q - 1, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q), seasonal = c(P - 1, D, Q - 1), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P - 1, D, Q - 1), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q, P - 1, D, Q - 1, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         Q <- (Q - 1)
         P <- (P - 1)
+        next
+      }
+    }
+    if (Q < max.Q && P > 0 && newmodel(p, d, q, P - 1, D, Q + 1, constant, results[1:k, ])) {
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P - 1, D, Q + 1), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+      results[k, ] <- c(p, d, q, P - 1, D, Q + 1, constant, fit$ic)
+      if (fit$ic < bestfit$ic) {
+        bestfit <- fit
+        Q <- (Q + 1)
+        P <- (P - 1)
+        next
+      }
+    }
+    if (Q > 0 && P < max.P && newmodel(p, d, q, P + 1, D, Q - 1, constant, results[1:k, ])) {
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P + 1, D, Q - 1), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+      results[k, ] <- c(p, d, q, P + 1, D, Q - 1, constant, fit$ic)
+      if (fit$ic < bestfit$ic) {
+        bestfit <- fit
+        Q <- (Q - 1)
+        P <- (P + 1)
+        next
       }
     }
     if (Q < max.Q && P < max.P && newmodel(p, d, q, P + 1, D, Q + 1, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q), seasonal = c(P + 1, D, Q + 1), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q), seasonal = c(P + 1, D, Q + 1), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q, P + 1, D, Q + 1, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         Q <- (Q + 1)
         P <- (P + 1)
+        next
       }
     }
 
     if (p > 0 && newmodel(p - 1, d, q, P, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p - 1, d, q), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p - 1, d, q), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p - 1, d, q, P, D, Q, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         p <- (p - 1)
-      }
-    }
-    if (p < max.p && newmodel(p + 1, d, q, P, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p + 1, d, q), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
-      results[k, ] <- c(p + 1, d, q, P, D, Q, constant, fit$ic)
-      if (fit$ic < bestfit$ic) {
-        bestfit <- fit
-        p <- (p + 1)
+        next
       }
     }
     if (q > 0 && newmodel(p, d, q - 1, P, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q - 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q - 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q - 1, P, D, Q, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         q <- (q - 1)
+        next
+      }
+    }
+    if (p < max.p && newmodel(p + 1, d, q, P, D, Q, constant, results[1:k, ])) {
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p + 1, d, q), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+      results[k, ] <- c(p + 1, d, q, P, D, Q, constant, fit$ic)
+      if (fit$ic < bestfit$ic) {
+        bestfit <- fit
+        p <- (p + 1)
+        next
       }
     }
     if (q < max.q && newmodel(p, d, q + 1, P, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p, d, q + 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p, d, q + 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p, d, q + 1, P, D, Q, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         q <- (q + 1)
+        next
       }
     }
     if (q > 0 && p > 0 && newmodel(p - 1, d, q - 1, P, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p - 1, d, q - 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p - 1, d, q - 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p - 1, d, q - 1, P, D, Q, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         q <- (q - 1)
         p <- (p - 1)
+        next
+      }
+    }
+    if (q < max.q && p > 0 && newmodel(p - 1, d, q + 1, P, D, Q, constant, results[1:k, ])) {
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p - 1, d, q + 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+      results[k, ] <- c(p - 1, d, q + 1, P, D, Q, constant, fit$ic)
+      if (fit$ic < bestfit$ic) {
+        bestfit <- fit
+        q <- (q + 1)
+        p <- (p - 1)
+        next
+      }
+    }
+    if (q > 0 && p < max.p && newmodel(p + 1, d, q - 1, P, D, Q, constant, results[1:k, ])) {
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p + 1, d, q - 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
+      results[k, ] <- c(p + 1, d, q - 1, P, D, Q, constant, fit$ic)
+      if (fit$ic < bestfit$ic) {
+        bestfit <- fit
+        q <- (q - 1)
+        p <- (p + 1)
+        next
       }
     }
     if (q < max.q && p < max.p && newmodel(p + 1, d, q + 1, P, D, Q, constant, results[1:k, ])) {
-      k <- k + 1
-      fit <- myarima(x, order = c(p + 1, d, q + 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+      k <- k + 1; if(k>nmodels) next
+      fit <- myarima(x, order = c(p + 1, d, q + 1), seasonal = c(P, D, Q), constant = constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
       results[k, ] <- c(p + 1, d, q + 1, P, D, Q, constant, fit$ic)
       if (fit$ic < bestfit$ic) {
         bestfit <- fit
         q <- (q + 1)
         p <- (p + 1)
+        next
       }
     }
     if (allowdrift || allowmean) {
       if (newmodel(p, d, q, P, D, Q, !constant, results[1:k, ])) {
-        k <- k + 1
-        fit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q), constant = !constant, ic, trace, approximation, offset = offset, xreg = xreg, ...)
+        k <- k + 1; if(k>nmodels) next
+        fit <- myarima(x, order = c(p, d, q), seasonal = c(P, D, Q), constant = !constant, ic, trace, approximation, method = method, offset = offset, xreg = xreg, ...)
         results[k, ] <- c(p, d, q, P, D, Q, !constant, fit$ic)
         if (fit$ic < bestfit$ic) {
           bestfit <- fit
@@ -534,6 +627,10 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     }
   }
 
+  if(k > nmodels){
+    warning(sprintf("Stepwise search was stopped early due to reaching the model number limit: `nmodels = %i`", nmodels))
+  }
+
   # Refit using ML if approximation used for IC
   if (approximation && !is.null(bestfit$arma)) {
     if (trace) {
@@ -541,14 +638,14 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
     }
     icorder <- order(results[, 8])
     nmodels <- sum(!is.na(results[, 8]))
-    for (i in 1:nmodels)
+    for (i in seq(nmodels))
     {
       k <- icorder[i]
       fit <- myarima(
         x, order = c(results[k, 1], d, results[k, 3]),
         seasonal = c(results[k, 4], D, results[k, 6]),
         constant = results[k, 7] == 1,
-        ic, trace, approximation = FALSE, xreg = xreg, ...
+        ic, trace, approximation = FALSE, method = method, xreg = xreg, ...
       )
       if (fit$ic < Inf) {
         bestfit <- fit
@@ -558,7 +655,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   }
 
   # Nothing fitted
-  if (bestfit$ic == Inf) {
+  if (bestfit$ic == Inf && !isTRUE(method=="CSS")) {
     if (trace) {
       cat("\n")
     }
@@ -573,7 +670,7 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   bestfit$call <- match.call()
   bestfit$call$x <- data.frame(x = x)
   bestfit$lambda <- lambda
-  bestfit$fitted <- fitted(bestfit)
+  bestfit$fitted <- fitted.Arima(bestfit)
 
   if (trace) {
     cat("\n\n Best model:", arima.string(bestfit, padding = TRUE), "\n\n")
@@ -582,19 +679,24 @@ auto.arima <- function(y, d=NA, D=NA, max.p=5, max.q=5,
   return(bestfit)
 }
 
-
 # Calls arima from stats package and adds data to the returned object
 # Also allows refitting to new data
 # and drift terms to be included.
-myarima <- function(x, order = c(0, 0, 0), seasonal = c(0, 0, 0), constant=TRUE, ic="aic", trace=FALSE, approximation=FALSE, offset=0, xreg=NULL, ...) {
-  n <- length(x)
+myarima <- function(x, order = c(0, 0, 0), seasonal = c(0, 0, 0), constant=TRUE, ic="aic", trace=FALSE, approximation=FALSE, offset=0, xreg=NULL, method = NULL, ...) {
+  # Length of non-missing interior
+  missing <- is.na(x)
+  firstnonmiss <- head(which(!missing),1)
+  lastnonmiss <- tail(which(!missing),1)
+  n <- lastnonmiss - firstnonmiss + 1
   m <- frequency(x)
   use.season <- (sum(seasonal) > 0) & m > 0
   diffs <- order[2] + seasonal[2]
-  if (approximation) {
-    method <- "CSS"
-  } else {
-    method <- "CSS-ML"
+  if(is.null(method)){
+    if (approximation) {
+      method <- "CSS"
+    } else {
+      method <- "CSS-ML"
+    }
   }
   if (diffs == 1 && constant) {
     xreg <- cbind(drift = 1:length(x), xreg)
@@ -625,7 +727,7 @@ myarima <- function(x, order = c(0, 0, 0), seasonal = c(0, 0, 0), constant=TRUE,
       fit$xreg <- xreg
     }
     npar <- length(fit$coef) + 1
-    if (approximation) {
+    if (method == "CSS") {
       fit$aic <- offset + nstar * log(fit$sigma2) + 2 * npar
     }
     if (!is.na(fit$aic)) {
@@ -651,10 +753,14 @@ myarima <- function(x, order = c(0, 0, 0), seasonal = c(0, 0, 0), constant=TRUE,
       }
       if (last.nonzero > 0) {
         testvec <- testvec[1:last.nonzero]
-        minroot <- min(minroot, abs(polyroot(c(1, -testvec))))
+        proots <- try(polyroot(c(1,-testvec)))
+        if (!is.element("try-error", class(proots))) {
+          minroot <- min(minroot, abs(proots))
+        }
+        else fit$ic <- Inf
       }
     }
-    if (order[3] + seasonal[3] > 0) {
+    if (order[3] + seasonal[3] > 0 & fit$ic < Inf) {
       testvec <- fit$model$theta
       k <- abs(testvec) > 1e-8
       if (sum(k) > 0) {
@@ -664,7 +770,11 @@ myarima <- function(x, order = c(0, 0, 0), seasonal = c(0, 0, 0), constant=TRUE,
       }
       if (last.nonzero > 0) {
         testvec <- testvec[1:last.nonzero]
-        minroot <- min(minroot, abs(polyroot(c(1, testvec))))
+        proots <- try(polyroot(c(1,testvec)))
+        if (!is.element("try-error", class(proots))) {
+          minroot <- min(minroot, abs(proots))
+        }
+        else fit$ic <- Inf
       }
     }
     if (minroot < 1 + 1e-2) { # Previously 1+1e-3
@@ -705,10 +815,11 @@ myarima <- function(x, order = c(0, 0, 0), seasonal = c(0, 0, 0), constant=TRUE,
 
 newmodel <- function(p, d, q, P, D, Q, constant, results) {
   n <- nrow(results)
-  for (i in 1:n)
-  {
-    if (identical(c(p, d, q, P, D, Q, constant), results[i, 1:7])) {
-      return(FALSE)
+  for (i in 1:n) {
+    if(!all(is.na(results[i, seq(7)]))) {
+      if (all(c(p, d, q, P, D, Q, constant) == results[i, 1:7])) {
+        return(FALSE)
+      }
     }
   }
   return(TRUE)
@@ -767,8 +878,6 @@ checkarima <- function(object) {
   return(test)
 }
 
-
-
 #' Is an object constant?
 #'
 #' Returns true if the object's numerical values do not vary.
