@@ -4,9 +4,6 @@
 # h is number of steps ahead to forecast
 # ... are passed to forecastfunction
 
-
-
-
 #' Time series cross-validation
 #'
 #' \code{tsCV} computes the forecast errors obtained by applying
@@ -30,6 +27,8 @@
 #' must have an argument \code{h} for the forecast horizon.
 #' @param h Forecast horizon
 #' @param window Length of the rolling window, if NULL, a rolling window will not be used.
+#' @param xreg Exogeneous predictor variables passed to the forecast function if required.
+#' @param initial Initial period of the time series where no cross-validation is performed.
 #' @param ... Other arguments are passed to \code{forecastfunction}.
 #' @return Numerical time series object containing the forecast errors as a vector (if h=1)
 #' and a matrix otherwise. The time index corresponds to the last period of the training
@@ -48,22 +47,44 @@
 #' e <- tsCV(lynx, far2, h=1, window=30)
 #'
 #' @export
-tsCV <- function(y, forecastfunction, h=1, window=NULL, ...) {
+tsCV <- function(y, forecastfunction, h=1, window=NULL, xreg=NULL, initial=0, ...) {
   y <- as.ts(y)
   n <- length(y)
   e <- ts(matrix(NA_real_, nrow = n, ncol = h))
+  if(initial >= n) stop("initial period too long")
   tsp(e) <- tsp(y)
-  for (i in seq_len(n-1))
-  {
-    fc <- try(suppressWarnings(
-      forecastfunction(subset(
-        y,
+  if (!is.null(xreg)) {
+    # Make xreg a ts object to allow easy subsetting later
+    xreg <- ts(as.matrix(xreg))
+    tsp(xreg) <- tsp(y)
+  }
+  if (is.null(window))
+    indx <- seq(1+initial, n - 1L)
+  else
+    indx <- seq(window+initial, n - 1L, by = 1L)
+  for (i in indx) {
+    y_subset <- subset(
+      y,
+      start = ifelse(is.null(window), 1L,
+              ifelse(i - window >= 0L, i - window + 1L, stop("small window"))
+      ),
+      end = i
+    )
+    if (is.null(xreg)) {
+      fc <- try(suppressWarnings(
+        forecastfunction(y_subset, h = h, ...)
+        ), silent = TRUE)
+    }
+    else {
+      xreg_subset <- as.matrix(subset(
+        xreg,
         start = ifelse(is.null(window), 1L,
-          ifelse(i - window >= 0L, i - window + 1L, stop("small window"))
-        ),
-        end = i
-      ), h = h, ...)
-    ), silent = TRUE)
+                ifelse(i - window >= 0L, i - window + 1L, stop("small window")))
+      ))
+      fc <- try(suppressWarnings(
+        forecastfunction(y_subset, h = h, xreg = xreg_subset, ...)
+        ), silent = TRUE)
+    }
     if (!is.element("try-error", class(fc))) {
       e[i, ] <- y[i + (1:h)] - fc$mean
     }
@@ -71,12 +92,12 @@ tsCV <- function(y, forecastfunction, h=1, window=NULL, ...) {
   if (h == 1) {
     return(e[, 1L])
   } else {
-    colnames(e) <- paste("h=",1:h,sep="")
+    colnames(e) <- paste("h=", 1:h, sep = "")
     return(e)
   }
 }
 
-## Cross-validation for AR models
+# Cross-validation for AR models
 # By Gabriel Caceres
 ## Note arguments to pass must be named
 
@@ -107,7 +128,7 @@ tsCV <- function(y, forecastfunction, h=1, window=NULL, ...) {
 #' @author Gabriel Caceres and Rob J Hyndman
 #' @seealso \link{CV}, \link{tsCV}.
 #' @references Bergmeir, C., Hyndman, R.J., Koo, B. (2018) A note on the
-#' validity of cross-validation for evaluating time series prediction. 
+#' validity of cross-validation for evaluating time series prediction.
 #' \emph{Computational Statistics & Data Analysis}, \bold{120}, 70-83.
 #' \url{https://robjhyndman.com/publications/cv-time-series/}.
 #' @keywords ts
@@ -126,7 +147,7 @@ tsCV <- function(y, forecastfunction, h=1, window=NULL, ...) {
 #' @export
 CVar <- function(y, k=10, FUN=nnetar, cvtrace=FALSE, blocked=FALSE, LBlags=24, ...) {
   nx <- length(y)
-  ## n-folds at most equal number of points
+  # n-folds at most equal number of points
   k <- min(as.integer(k), nx)
   if (k <= 1L) {
     stop("k must be at least 2")
@@ -142,8 +163,7 @@ CVar <- function(y, k=10, FUN=nnetar, cvtrace=FALSE, blocked=FALSE, LBlags=24, .
   cvacc <- matrix(NA_real_, nrow = k, ncol = 7)
   out <- list()
   alltestfit <- rep(NA, length.out = nx)
-  for (i in 1:k)
-  {
+  for (i in 1:k) {
     out[[paste0("fold", i)]] <- list()
     testset <- ind[fold == i]
     trainset <- ind[fold != i]
@@ -174,10 +194,12 @@ CVar <- function(y, k=10, FUN=nnetar, cvtrace=FALSE, blocked=FALSE, LBlags=24, .
   out$LBpvalue <- Box.test(out$residuals, type = "Ljung", lag = LBlags)$p.value
 
   out$k <- k
-  ## calculate mean accuracy accross all folds
-  CVmean <- matrix(apply(cvacc, 2, FUN = mean, na.rm = TRUE), dimnames = list(colnames(acc), "Mean"))
-  ## calculate accuracy sd accross all folds --- include?
-  CVsd <- matrix(apply(cvacc, 2, FUN = sd, na.rm = TRUE), dimnames = list(colnames(acc), "SD"))
+  # calculate mean accuracy accross all folds
+  CVmean <- matrix(apply(cvacc, 2, FUN = mean, na.rm = TRUE),
+    dimnames = list(colnames(acc), "Mean"))
+  # calculate accuracy sd accross all folds --- include?
+  CVsd <- matrix(apply(cvacc, 2, FUN = sd, na.rm = TRUE),
+    dimnames = list(colnames(acc), "SD"))
   out$CVsummary <- cbind(CVmean, CVsd)
   out$series <- deparse(substitute(y))
   out$call <- match.call()
@@ -189,12 +211,12 @@ print.CVar <- function(x, ...) {
   cat("Series:", x$series, "\n")
   cat("Call:   ")
   print(x$call)
-  ## Add info about series, function, and parameters
-  ## Add note about any NA/NaN in folds?
-  ##
-  ## Print number of folds
+  # Add info about series, function, and parameters
+  # Add note about any NA/NaN in folds?
+  #
+  # Print number of folds
   cat("\n", x$k, "-fold cross-validation\n", sep = "")
-  ## Print mean & sd accuracy() results
+  # Print mean & sd accuracy() results
   print(x$CVsummary)
 
   cat("\n")
